/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.camel.builder

import java.util.concurrent.atomic.AtomicInteger
import metridoc.camel.iterator.IteratorCreator
import org.apache.camel.Exchange
import org.apache.camel.Processor
import org.apache.camel.component.seda.SedaEndpoint
import metridoc.camel.aggregator.BodyAggregator
import metridoc.camel.aggregator.InflightAggregationWrapper
import metridoc.camel.impl.iterator.IteratorProviderFactory

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 8/26/11
 * Time: 2:02 PM
 * To change this template use File | Settings | File Templates.
 */
class BatchLoaderBuilder extends ManagedRouteBuilder {

    Map<Throwable, Closure> validationErrors
    private static AtomicInteger sedaId;
    SedaEndpoint sedaEndpoint
    IteratorCreator iteratorCreator
    int queueSize
    int threads
    int batchSize
    long completionTimeout = 3000
    Closure parse

    /**
     * pause if queue has reached size limit
     */
    long pause = 100
    String from
    String to

    @Override
    void doConfigure() {
        def sedaEndpoint =
            getContext().getEndpoint("seda:batchLoader-${sedaId.getAndIncrement()}?concurrentConsumers=${threads}")
        def batchThrottler = new BatchThrottler(batchLoaderBuilder: this)

        def iteratorCreatorToUse

        if (iteratorCreator) {
            iteratorCreatorToUse = iteratorCreator
        } else {
            iteratorCreatorToUse = IteratorProviderFactory.class
        }

        from(from).split().method(iteratorCreatorToUse, "create").process(batchThrottler).to(sedaEndpoint)

        from(sedaEndpoint).process(parse).aggregate(constant(true), new InflightAggregationWrapper(new BodyAggregator())).completionSize(batchSize).completionTimeout(completionTimeout).to(to)

    }

    Throwable getFirstException() {
        return null  //To change body of implemented methods use File | Settings | File Templates.
    }
}

class BatchThrottler implements Processor {
    BatchLoaderBuilder batchLoaderBuilder

    void process(Exchange exchange) {
        def queue = batchLoaderBuilder.sedaEndpoint.queue
        def queueSize = batchLoaderBuilder.queueSize
        def pause = batchLoaderBuilder.pause

        if (queue.size() >= queueSize) {
            Thread.sleep(pause)
        }
    }
}

